package com.mybank;

import static org.hamcrest.MatcherAssert.*;
import static org.hamcrest.Matchers.*;
import static org.mockito.Matchers.*;
import static org.mockito.Mockito.*;

import org.junit.Test;

import com.mybank.AccountService.PostWithdrawalCheck;
import com.mybank.builders.AccountMother;

public class AccountServiceTest {

	@Test
	public void shouldTakeMoneyOutOfAccount() throws Exception {
		AccountService service = new AccountService();
		Account account = new AccountMother().janesAccount();
		account.depositMoney(120);
		
		service.withdrawMoney(account, 100);
		
		assertThat(account.viewBalance(), is(20));
	}
	
	@Test
	public void shouldTakeMoneyOutOfAccount_withDynamicMock() throws Exception {
		AccountService service = new AccountService();
		Account account = mock(Account.class);
		
		service.withdrawMoney(account, 333);

		verify(account).withdrawMoney(333);
	}
	
	@Test
	public void shouldCarryOutChecksAfterWithdrawal() throws Exception {

		PostWithdrawalCheck check1 = mock(PostWithdrawalCheck.class);
		PostWithdrawalCheck check2 = mock(PostWithdrawalCheck.class);
		AccountService service = new AccountService(check1, check2);
		Account account = new AccountMother().janesAccount();
		
		service.withdrawMoney(account, 555);
		
		verify(check1).check(same(account), eq(555));
		verify(check2).check(same(account), eq(555));
	}

}
