package com.mybank;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.equalTo;
import static org.hamcrest.Matchers.is;
import static org.junit.Assert.*;

import org.junit.Before;
import org.junit.Test;

import com.mybank.builders.AccountBuilder;

public class AccountTest {

	private Account account;

	@Before
	public void creatEmptyWallet() {
		account = new Account();
	}

	@Test
	public void shouldBeAbleToPutMoneyInAndTakeItOutLater() throws Exception {
		account.depositMoney(5645645);
		assertThat(account.withdrawMoney(), is(equalTo(5645645)));

		Account secondAccount = new Account();
		secondAccount.depositMoney(34324);
		assertThat(secondAccount.withdrawMoney(), is(equalTo(34324)));
	}
	
	@Test
	public void shouldAddDepositToBalance() throws Exception {
		Account account = new AccountBuilder().withBalance(0).build();
		
		account.depositMoney(131);
		assertThat(account.viewBalance(), is(131));
		
		account.depositMoney(28);
		assertThat(account.viewBalance(), is(159));
	}

	@Test
	public void shouldNotBeAbleToTakeOutMoneyMoreThanOnce() throws Exception {
		account.depositMoney(12);
		assertThat(account.withdrawMoney(), is(equalTo(12)));
		assertThat(account.withdrawMoney(), is(equalTo(0)));
	}

	@Test
	public void shouldShowHowMuchMoneyItHasWithoutAffectingTheAmount()
			throws Exception {
		account.depositMoney(234);
		assertThat(account.viewBalance(), is(234));
		assertThat(account.viewBalance(), is(234));
	}

	@Test
	public void shouldAllowTakingOutASpecifiedAmount() throws Exception {
		Account account = new AccountBuilder().withBalance(567).build();
		assertThat(account.viewBalance(), is(567));

		int taken = account.withdrawMoney(67);
		assertThat(taken, is(67));
		assertThat(account.viewBalance(), is(500));
	}
	
	@Test
	public void shouldAllowTakingOutMoreThanWasPutIn() throws Exception {
		account.depositMoney(80);
		int taken = account.withdrawMoney(100);
		assertThat(taken, is(100));
		assertThat(account.viewBalance(), is(-20));
	}

	@Test
	public void shouldTellWhenItsOverdrawn() throws Exception {
		account.depositMoney(5);
		assertThat(account.isOverdrawn(), is(false));
		
		account.withdrawMoney(10);
		assertThat(account.isOverdrawn(), is(true));
	}
	
}
